﻿using Microsoft.Samples.ServiceHosting.StorageClient;
using System;
using System.IO;
using System.Net;
using System.Security.Cryptography;
using System.Text;

namespace ContainerACLTest
{
    class Program
    {
        public static string MakePolicySignature(string account, string key, string container, string policyId)
        {
            string stringtosign = string.Format("\n\n\n/{0}/{1}\n{2}", account, container, policyId);
            using (var hmac = new HMACSHA256(Convert.FromBase64String(key)))
            {
                return Convert.ToBase64String(hmac.ComputeHash(Encoding.UTF8.GetBytes(stringtosign)));
            }
        }

        static void Main(string[] args)
        {
            var account = "GETYOUROWNACCOUNT";
            var key = "GETYOUROWNKEY";
            var containername = "testcontainer";
            var blobname = "testblob";
            var policyname = "testid";
            var permissions = "r";
            var yesterday = (DateTime.Today.ToUniversalTime() - TimeSpan.FromDays(1)).ToString("yyyy-MM-dd");
            var tomorrow = (DateTime.Today.ToUniversalTime() + TimeSpan.FromDays(1)).ToString("yyyy-MM-dd");
            var skc = new SharedKeyCredentials(account, Convert.FromBase64String(key));
            Console.WriteLine("Creating container...");
            var container = BlobStorage.Create(new Uri("http://blob.core.windows.net"), null, account, key).GetBlobContainer(containername);
            container.CreateContainer();
            Console.WriteLine("Container created.");
            Console.WriteLine("Creating blob...");
            container.CreateBlob(new BlobProperties(blobname) { ContentType = "text/plain" }, new BlobContents(Encoding.UTF8.GetBytes("Hello, World!")), true);
            Console.WriteLine("Blob created.");
            Console.WriteLine("Creating access policy...");
            var req = (HttpWebRequest)(WebRequest.Create(
                string.Format("http://{0}.blob.core.windows.net/{1}?restype=container&comp=acl", account, containername)));
            req.Method = "PUT";
            req.Headers.Add("x-ms-version", "2009-07-17");
            req.Headers.Add("x-ms-date", DateTime.UtcNow.ToString("R"));
            req.Headers.Add("x-ms-prop-publicaccess", "false");
            var body = string.Format(@"<?xml version=""1.0"" encoding=""utf-8""?>
                                        <SignedIdentifiers>
                                            <SignedIdentifier>
                                                <Id>{0}</Id>
                                                <AccessPolicy>
                                                    <Start>{1}</Start>
                                                    <Expiry>{2}</Expiry>
                                                    <Permission>{3}</Permission>
                                                </AccessPolicy>
                                            </SignedIdentifier>
                                        </SignedIdentifiers>", policyname, yesterday, tomorrow, permissions);
            req.ContentLength = body.Length;
            skc.SignRequest(req, new ResourceUriComponents(account, containername, "comp=acl"));
            using (var stream = new StreamWriter(req.GetRequestStream()))
            {
                stream.Write(body);
                stream.Close();
            }
            req.GetResponse();
            Console.WriteLine("Access policy created.");
            Console.WriteLine("Reading blob...");
            var sig = MakePolicySignature(account, key, containername, policyname);
            string text = null;
            using (var reader = 
                new StreamReader(
                    WebRequest.Create(
                        string.Format("http://{0}.blob.core.windows.net/{1}/{2}?sr=c&si={3}&sig={4}",
                            account,
                            containername,
                            blobname,
                            policyname,
                            Uri.EscapeDataString(sig)
                        )
                    ).GetResponse().GetResponseStream()))
            {
                text = reader.ReadToEnd();
            }
            Console.WriteLine("Got blob text: '{0}'", text);
        }
    }
}